@@ -1,2 +1,6 @@
 from .view import nwb2widget, default_neurodata_vis_spec
-#from .ephys_viz_interface import ephys_viz_neurodata_vis_spec
\ No newline at end of file
+#from .ephys_viz_interface import ephys_viz_neurodata_vis_spec
+
+import plotly.io as pio
+
+pio.templates.default = "simple_white"
